#include <util/twi.h>



////////////////////////////////////////////////////////////
// Interne-Routinen:
////////////////////////////////////////////////////////////

/* 	Gibt 1 zurck, falls TWINT gesetzt wird.
	Falls nach 40000 Schleifendurchgngen TWINT
	noch nicht gesetzt wurde, gibt 0 zurck.
	*/	
int twi_wait4TWINT() {
	uint16_t x=0;
	while (x<40000) { 
		x++; 
		if ( TWCR & (1<<TWINT) ) {	return 1; }		
	}
	return 0;
}

/*	Gibt 0 zurck, falls der Parameter status nicht
	in den oberen 5 Bytes von TWSR zu finden ist. 
	Sonst, wenn alles in Ordnung ist 1. 
	*/
int twi_check( uint8_t status ) {
	return ((TWSR & 0xF8) == status );
}


////////////////////////////////////////////////////////////
// Init-, Start- und Stop-Routinen:
////////////////////////////////////////////////////////////

/* 	Setzt die Geschwindikeit des Busses abhngig von der 
	CPU-Geschwindigkeit. */
void twi_init() {

	TWBR = 2;
	TWSR = 0;
	//==> SLCfreq==50KHz bei 1MHz CPU-Takt
}


/*	Leitet einen TWI (I2C) Sende- oder Empfangsvorgang ein.
	Falls erfolgreich, gibt 1 zurck sonst 0; */
int twi_start() {

	// Setze Start-Condition:
	TWCR = (1<<TWINT)|(1<<TWSTA)|(1<<TWEN);

	// Warte auf das TWINT-Flag:
	twi_wait4TWINT();

	// Gebe den Status zurck:
	return twi_check(TW_START);
}


int twi_rep_start(){
	// Setze Start-Condition:
	TWCR = (1<<TWINT)|(1<<TWSTA)|(1<<TWEN);

	// Warte: auf das TWINT-Flag:
	twi_wait4TWINT();

	// Gebe den Status zurck:
	return twi_check(TW_REP_START);
}


/*	Beendet einen TWI (I2C) Sende- oder Empfangsvorgang. */
void twi_stop(){
	TWCR = (1<<TWINT)|(1<<TWSTO)|(1<<TWEN);
}


////////////////////////////////////////////////////////////
// Sende-Routinen:
////////////////////////////////////////////////////////////

int twi_sla_w(uint8_t slave_adress){

	TWDR = slave_adress;					// Slave Adresse in das Data Register schreiben
	TWCR = (1<<TWINT)|(1<<TWEN);			// Adresse senden.

	// Warten auf das TWINT-Flag
	twi_wait4TWINT();


	return twi_check(TW_MT_SLA_ACK);
}

int twi_sla_r(uint8_t slave_adress){

	TWDR = slave_adress;					// Slave Adresse in das Data Register schreiben
	TWCR = (1<<TWINT)|(1<<TWEN);			// Adresse senden.

	// Warten auf das TWINT-Flag
	twi_wait4TWINT();

	return twi_check(TW_MR_SLA_ACK);
}

int twi_send(uint8_t data){

	TWDR = data;							// Datum in das Data Register schreiben
	TWCR = (1<<TWINT)|(1<<TWEN);			// Datum Senden.

	// Warten auf das TWINT-Flag
	twi_wait4TWINT();

	return twi_check(TW_MT_DATA_ACK);
}


////////////////////////////////////////////////////////////
// Empfangs-Routine:
////////////////////////////////////////////////////////////

/*	Liest ein Byte vom Bus. 

	Parameter:
	int ack 		- ACK-Bit
	uint8_t* data	- Unter *data wird das gelesene Byte gespeichert

	Falls ack!=0 wird der Empfang mit einem	ACK-Bit besttigt 
	(== Erwarte nchstes Byte). Sonst wird das Bit nicht gesetzt 
	(== Transmission beendet).

	Rckgabewert:
	Die Methode gibt eine 0 zurck falls ein Fehler auftritt. Sonst eine 1.
	Ein Fehler tritt auf wenn:
		- data==0 (null-pointer)
		- Lesen nicht erfolgreich war.

	Falls das Lesen nicht erfolgreich war, wird *data auf 0x00 gesetzt.
	
*/
int twi_read(int ack, uint8_t* data){

	// Falls data ein 0-Pointer ist, beende:
	if (data==0) { return 0; }

	// Starte den Lesevorgang:
	if (ack) {								// ACK Senden
		TWCR = (1<<TWINT)|(1<<TWEN)|(1<<TWEA);
	} else {								// NACK Senden
		TWCR = (1<<TWINT)|(1<<TWEN);
	}

	// Warten auf das TWINT-Flag
	twi_wait4TWINT();

	// berprfe den Status des Lesevorgangs:
	int status = 0;
	if (ack) { 	
		status = twi_check(TW_MR_DATA_ACK); 
	} else {
		status = twi_check(TW_MR_DATA_NACK);
	}

	// Falls lesen erfolgreich, bergebe es an die
	// *data-Variable. Sonst setze *data auf 0:
	if (status) {
		// Lese das Datenregister:
		*data = TWDR;
	} else {
		*data = 0;
	}


	// gebe den Status zurck:
	return status;

}

